from bulma.tests.forms import FormExample
from bulma.tests.utils import render_template


def test_bulma_base_template():
    output = render_template(
        """
        {% extends 'bulma/base.html' %}
        {% block content %}
            test_bulma_content
        {% endblock content %}
        """
    )

    assert "test_bulma_content" in output, "Content is in template"


def test_style_css_is_in_template():
    output = render_template(
        """
        {% extends 'bulma/base.html' %}
        """
    )
    assert '<link rel="stylesheet" href="/static/bulma/css/style.min.css">' in output


def test_bulma_form_tag():
    output = render_template(
        """
        {% extends 'bulma/base.html' %}
        {% load bulma_tags %}
        {% block content %}
            <form>{{ form|bulma }}</form>
        {% endblock content %}
        """, context={
            'form': FormExample()
        }
    )

    assert '<div class="field"' in output, "Fields are rendered"



from django import forms
from django import template
from django.forms import BoundField
from django.template.loader import get_template
from django.utils.safestring import mark_safe

register = template.Library()

BULMA_COLUMN_COUNT = 1


@register.simple_tag
def font_awesome():
    """
    The latest FontAwesome CDN link.
    """
    cdn_link = (
        '<link rel="stylesheet" '
        'href="https://use.fontawesome.com/releases/v5.7.2/css/all.css" '
        'integrity="sha384-fnmOCqbTlWIlj8LyTjo7mOUStjsKC4pOpQbqyi7RrhN7udi9RwhKkMHpvLbHG9Sr" '
        'crossorigin="anonymous">'
    )
    return mark_safe(cdn_link)


@register.filter
def bulma(element):
    markup_classes = {'label': '', 'value': '', 'single_value': ''}
    return render(element, markup_classes)


@register.filter
def bulma_inline(element):
    markup_classes = {'label': 'sr-only', 'value': '', 'single_value': ''}
    return render(element, markup_classes)


@register.filter
def bulma_horizontal(element, label_cols='is-2'):
    markup_classes = {'label': label_cols, 'value': '', 'single_value': ''}

    for cl in label_cols.split(' '):
        splitted_class = cl.split('-')

        try:
            value_nb_cols = int(splitted_class[-1])
        except ValueError:
            value_nb_cols = BULMA_COLUMN_COUNT

        if value_nb_cols >= BULMA_COLUMN_COUNT:
            splitted_class[-1] = str(BULMA_COLUMN_COUNT)
        else:
            offset_class = cl.split('-')
            offset_class[-1] = 'offset-' + str(value_nb_cols)
            splitted_class[-1] = str(BULMA_COLUMN_COUNT - value_nb_cols)
            markup_classes['single_value'] += ' ' + '-'.join(offset_class)
            markup_classes['single_value'] += ' ' + '-'.join(splitted_class)

        markup_classes['value'] += ' ' + '-'.join(splitted_class)

    return render(element, markup_classes)


@register.filter
def add_input_classes(field):
    if not is_checkbox(field) and not is_multiple_checkbox(field) \
            and not is_radio(field) and not is_file(field):
        field_classes = field.field.widget.attrs.get('class', '')
        field_classes += ' control'
        field.field.widget.attrs['class'] = field_classes


def render(element, markup_classes):
    if isinstance(element, BoundField):
        add_input_classes(element)
        template = get_template("bulma/forms/field.html")
        context = {'field': element,
                   'classes': markup_classes, 'form': element.form}
    else:
        has_management = getattr(element, 'management_form', None)
        if has_management:
            for form in element.forms:
                for field in form.visible_fields():
                    add_input_classes(field)

            template = get_template("bulma/forms/formset.html")
            context = {'formset': element, 'classes': markup_classes}
        else:
            for field in element.visible_fields():
                add_input_classes(field)

            template = get_template("bulma/forms/form.html")
            context = {'form': element, 'classes': markup_classes}

    return template.render(context)


@register.filter
def widget_type(field):
    return field.field.widget


@register.filter
def is_select(field):
    return isinstance(field.field.widget, forms.Select)


@register.filter
def is_multiple_select(field):
    return isinstance(field.field.widget, forms.SelectMultiple)


@register.filter
def is_textarea(field):
    return isinstance(field.field.widget, forms.Textarea)


@register.filter
def is_input(field):
    return isinstance(field.field.widget, (
        forms.TextInput,
        forms.NumberInput,
        forms.EmailInput,
        forms.PasswordInput,
        forms.URLInput
    ))


@register.filter
def is_checkbox(field):
    return isinstance(field.field.widget, forms.CheckboxInput)


@register.filter
def is_multiple_checkbox(field):
    return isinstance(field.field.widget, forms.CheckboxSelectMultiple)


@register.filter
def is_radio(field):
    return isinstance(field.field.widget, forms.RadioSelect)


@register.filter
def is_file(field):
    return isinstance(field.field.widget, forms.FileInput)


@register.filter
def addclass(field, css_class):
    if len(field.errors) > 0:
        css_class += ' is-danger'
    field_classes = field.field.widget.attrs.get('class', '')
    field_classes += f' {css_class}'
    return field.as_widget(attrs={"class": field_classes})



@register.filter
def bulma_message_tag(tag):
    return {
        'error': 'danger'
    }.get(tag, tag)



import os
import shutil
from django.conf import settings
from django.core.management import BaseCommand, CommandError


class Command(BaseCommand):

    help = 'Copies bulma static files into project\'s main static dir'
    static_root_bulma_dir = None

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)

    def handle(self, *args, **options):
        self.validate()
        self.copy_bulma_files()

    def validate(self):

        if len(settings.STATICFILES_DIRS) == 0:
            raise CommandError(
                "STATICFILES_DIRS in your settings is empty. "
                "STATICFILES_DIRS should have at least one directory. "
                "Bulma static files will be put into first directory listed in STATICFILES_DIRS "
                "It's a good idea to set first item of STATICFILES_DIRS to os.path.join(BASE_DIR, \"static\")"
            )

        self.static_root_bulma_dir = os.path.join(settings.STATICFILES_DIRS[0], 'bulma')

        if os.path.exists(self.static_root_bulma_dir):
            result = input("'bulma' dir already exists in your first STATICFILES_DIRS directory, "
                           "do you want to overwrite its contents and continue? y/N: ")
            if result.lower() != 'y':
                raise CommandError('Command aborted')
            shutil.rmtree(self.static_root_bulma_dir)

    def copy_bulma_files(self):
        """
        Copies Bulma static files from package's static/bulma into project's
        STATIC_ROOT/bulma
        """
        original_bulma_dir = os.path.join(
            os.path.dirname(os.path.dirname(os.path.dirname(__file__))),
            'static',
            'bulma'
        )
        shutil.copytree(original_bulma_dir, self.static_root_bulma_dir)


import os
from io import StringIO

import pytest
from django.core.management import call_command, CommandError

from django.conf import settings

BULMA_STATIC_DIR = os.path.join(settings.STATICFILES_DIRS[0], 'bulma')


def test_bulma_command_without_second_argument_displays_command_error_message():
    out = StringIO()
    with pytest.raises(CommandError) as err:
        call_command('bulma', stdout=out)

    assert 'Command argument is missing, please add one of the following' in str(err.value)


def test_copy_static_content(cleanup_static_files):
    style_css = os.path.join(BULMA_STATIC_DIR, 'css', 'style.css')
    style_sass = os.path.join(BULMA_STATIC_DIR, 'sass', 'style.sass')
    package_json = os.path.join(BULMA_STATIC_DIR, 'sass', 'package.json')

    out = StringIO()
    call_command('copy_bulma_static_into_project', stdout=out)

    assert os.path.isfile(style_css), f'{str(style_css)} has been created'
    assert os.path.isfile(style_sass), f'{str(style_sass)} has been created'
    assert os.path.isfile(package_json), f'{str(package_json)} has been created'


def test_bulma_install_command(cleanup_static_files):
    call_command('copy_bulma_static_into_project')

    out = StringIO()
    call_command('bulma', 'install', stdout=out)

    assert os.path.isdir(os.path.join(BULMA_STATIC_DIR, 'sass', 'node_modules')), "node_modules have been created"
    assert os.path.isfile(
        os.path.join(BULMA_STATIC_DIR, 'sass', 'package-lock.json')), "package-lock.json has been created"


def test_bulma_build_command(cleanup_static_files):
    style_css = os.path.join(BULMA_STATIC_DIR, 'css', 'style.css')
    style_min_css = os.path.join(BULMA_STATIC_DIR, 'css', 'style.min.css')

    call_command('copy_bulma_static_into_project')

    # Delete preinstalled files
    os.remove(style_css)
    os.remove(style_min_css)

    call_command('bulma', 'install')

    out = StringIO()
    call_command('bulma', 'build', stdout=out)

    # Ensure new file have been generated
    assert os.path.isfile(style_css), f'{str(style_css)} has been created'
    assert os.path.isfile(style_min_css), f'{str(style_min_css)} has been created'



from django import forms

COLOR_CHOICES = (
    ('red', 'Red'),
    ('green', 'Green'),
    ('blue', 'Blue')
)


class FormExample(forms.Form):
    text = forms.CharField()
    email = forms.EmailField()
    number = forms.CharField(widget=forms.NumberInput())
    url = forms.CharField(widget=forms.URLInput())
    password = forms.CharField(widget=forms.PasswordInput())
    select = forms.ChoiceField(choices=COLOR_CHOICES)
    multi_select = forms.MultipleChoiceField(choices=COLOR_CHOICES)
    textarea = forms.CharField(widget=forms.Textarea())
    checkbox = forms.BooleanField()
    checkboxes = forms.MultipleChoiceField(
        choices=COLOR_CHOICES,
        widget=forms.CheckboxSelectMultiple()
    )
    radios = forms.ChoiceField(
        choices=COLOR_CHOICES,
        widget=forms.RadioSelect()
    )
    file = forms.FileField(required=True)



import os
import pytest
import shutil
from django.conf import settings


@pytest.yield_fixture
def cleanup_static_files():
    static_files_dir = settings.STATICFILES_DIRS[0]
    if os.path.isdir(static_files_dir):
        shutil.rmtree(static_files_dir)
    yield
    if os.path.isdir(static_files_dir):
        shutil.rmtree(static_files_dir)



import pytest
from django import forms

from .utils import render_form, get_dom, element_has_all_attributes

COLOR_CHOICES = (
    ('red', 'Red'),
    ('green', 'Green'),
    ('blue', 'Blue')
)


@pytest.mark.parametrize("field,label,tag,attributes", [
    (forms.CharField(), "Name", 'input', {
        'name': 'input',
        'class': ['control', 'input'],
        'type': 'text'
    }),
    (forms.EmailField(), "Email", 'input', {
        'name': 'input',
        'class': ['control', 'input'],
        'type': 'email'
    }),
    (forms.CharField(widget=forms.NumberInput()), "Email", 'input', {
        'name': 'input',
        'class': ['control', 'input'],
        'type': 'number'
    }),
    (forms.CharField(widget=forms.URLInput()), "Url", 'input', {
        'name': 'input',
        'class': ['control', 'input'],
        'type': 'url'
    }),
    (forms.CharField(widget=forms.PasswordInput()), "Password", 'input', {
        'name': 'input',
        'class': ['control', 'input'],
        'type': 'password'
    }),
    (forms.ChoiceField(choices=[]), "Select", 'select', {
        'name': 'input',
        'class': ['control']
    }),
    (forms.MultipleChoiceField(choices=[]), "Multi select", 'select', {
        'name': 'input',
        'class': ['control'],
        'multiple': ""
    }),
    (forms.CharField(widget=forms.Textarea()), "Textarea", 'textarea', {
        'name': 'input',
        'class': ['control', 'textarea']
    }),
    (forms.BooleanField(), "Checkbox", 'input', {
        'name': 'input',
        'type': 'checkbox'
    }),
    (forms.MultipleChoiceField(
        choices=COLOR_CHOICES,
        widget=forms.CheckboxSelectMultiple()
    ), "Checkboxes", 'input', {
         'name': 'input',
         'type': 'checkbox',
         'value': 'red'
     }),
    (forms.ChoiceField(
        choices=COLOR_CHOICES,
        widget=forms.RadioSelect()
    ), "Radios", 'input', {
         'name': 'input',
         'type': 'radio',
         'value': 'red'
     }),
    (forms.FileField(), "File input", 'input', {
        'name': 'input',
        'type': 'file',
        'class': ['file-input']
    })
])
def test_input_rendering(field, tag, label, attributes):
    class TestForm(forms.Form):
        input = field

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.fields['input'].label = label

    output = render_form(TestForm())
    dom = get_dom(output)

    element_has_all_attributes(dom.find(tag), attributes), f"{label} has attributes {str(attributes)}"
    assert dom.find('label').text.strip() == label, f"Field has label {label}"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bulma',
 'bulma.management',
 'bulma.management.commands',
 'bulma.templatetags',
 'bulma.tests']

package_data = \
{'': ['*'],
 'bulma': ['static/bulma/css/*',
           'static/bulma/sass/*',
           'templates/*',
           'templates/_layouts/*',
           'templates/bulma/*',
           'templates/bulma/forms/*',
           'templates/registration/*']}

install_requires = \
['django>=2.2,<3.1']

setup_kwargs = {
    'name': 'django-bulma',
    'version': '0.7.1',
    'description': 'Bulma CSS Framework for Django projects',
    'long_description': '# A Bulma Theme for Django Projects\n\n![Django Bulma](https://raw.githubusercontent.com/timonweb/django-bulma/master/test_project/static/images/django-bulma-logo.png)\n\nA Django base theme based on Bulma ([bulma.io](https://bulma.io/)). Bulma is a modern CSS framework based on Flexbox.\n\n*** work in progress ***\n\n## Installation\n\n1. Install the python package django-bulma from pip\n\n  ``pip install django-bulma``\n\n  Alternatively, you can install download or clone this repo and call ``pip install -e .``.\n\n2. Add to INSTALLED_APPS in your **settings.py**:\n\n  `\'bulma\',`\n\n3. If you want to use the provided base template, extend from **bulma/base.html**:\n\n  ```\n  {% extends \'bulma/base.html\' %}\n\n  {% block title %}Bulma Site{% endblock %}\n\n  {% block content %}\n    Content goes here...\n  {% endblock content %}\n\n  ```\n  \n4. If you want to customize bulma sass and your own components:\n\n    4.1 Copy bulma static files into your project\'s **STATIC_ROOT**:\n\n    ```\n    python manage.py copy_bulma_static_into_project\n    ```  \n    You should see **bulma** dir appeared in your **STATIC_ROOT**. It contains\n    two dirs:\n    * **sass** - this is the place where you can put your own sass code and customize\n    bulma variables\n    * **css** - this is where compiled sass output goes, you should link this file\n    in your base.html \n\n    4.2 Install npm packages for sass compilation to work:    \n    \n    ```\n    python manage.py bulma install\n    ```\n    \n    4.3 Start sass watch mode:\n    ```\n    python manage.py bulma start\n    ```\n\n5. For forms, in your templates, load the `bulma_tags` library and use the `|bulma` filters:\n\n    ##### Example template\n    \n    ```django\n\n    {% load bulma_tags %}\n\n    {# Display a form #}\n\n    <form action="/url/to/submit/" method="post">\n       {% csrf_token %}\n       {{ form|bulma }}\n       <div class="field">\n         <button type="submit" class="button is-primary">Login</button>\n       </div>\n       <input type="hidden" name="next" value="{{ next }}"/>\n    </form>\n    ```\n\n## Included templates\n\n**django-bulma** comes with:\n* a base template,\n* django core registration templates,\n\n## Bugs and suggestions\n\nIf you have found a bug or if you have a request for additional functionality, please use the issue tracker on GitHub.\n\n[https://github.com/timonweb/django-bulma/issues](https://github.com/timonweb/django-bulma/issues)\n',
    'author': 'Tim Kamanin',
    'author_email': 'tim@timonweb.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://timonweb.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<3.9',
}


setup(**setup_kwargs)



from bs4 import BeautifulSoup
from django.template import engines


def render_template(text, context=None):
    """
    Render template from string
    """
    template = engines["django"].from_string(text)
    if not context:
        context = {}
    return template.render(context)


def render_form(form):
    return render_template(
        """
        {% extends 'bulma/base.html' %}
        {% load bulma_tags %}
        
        {% block content %}
          <form method="post" enctype="multipart/form-data" action="." novalidate>
            {% csrf_token %}
            {{ form|bulma }}
            <button class="button is-primary">Submit</button>
          </form>
        {% endblock %}
        """, context={
            'form': form
        }
    )


def get_dom(html):
    return BeautifulSoup(html, 'html.parser')


def element_has_all_attributes(element, attributes):
    for attribute_name, attribute_value in attributes.items():
        assert element.has_attr(attribute_name) is True
        print(element.get(attribute_name))
        assert element.get(attribute_name) == attribute_value, f'Element {element} has attribute "{attribute_name}" with value {attribute_value}'
            #return False
    #return True


def element_has_attribute(element, attribute_name, attribute_value):
    return element.has_attr(attribute_name) and element.get(attribute_name) == attribute_value
